# This is inconsistent for each type of dataset we can have.
#
# class TestDatasetHashing(TestCase):
#     def setUp(self) -> None:
#         self.orig = pd.DataFrame(
#             [[1, 2, 3], [2, 3, 4], [3, 4, 5], [4, 5, 6]], columns=["a", "b", "c"]
#         )
#
#     def test_shuffle_rows(self):
#         shuf = self.orig.copy(deep=True)
#         shuf = shuf.reindex(np.random.permutation(len(shuf)))
#
#         self.assertEqual(
#             dataset_hash(self.orig, 1), dataset_hash(shuf, 1), "Row order affects hash"
#         )
#
#     def test_shuffle_columns(self):
#         shuf = self.orig.copy(deep=True)
#         shuf = shuf[[c for c in shuf if c not in ["c"]] + ["c"]]
#
#         self.assertEqual(
#             dataset_hash(self.orig, 1), dataset_hash(shuf, 1), "Column order affects hash"
#         )
#
#     def test_single_cell_change(self):
#         shuf = self.orig.copy(deep=True)
#         shuf.iat[0, 2] = 6
#         print(shuf)
#
#         self.assertNotEqual(
#             dataset_hash(self.orig, 1), dataset_hash(shuf, 1), "Row order affects hash"
#         )



import json
import os
from pathlib import Path


def ensure_path_exists(path):
    if not isinstance(path, str):
        raise TypeError(f"path must be of type str:{path}")
    if not os.path.exists(path):
        Path(path).mkdir(parents=True, exist_ok=True)


def to_file(content, path, mode="w+"):
    with open(path, mode) as f:
        f.write(content)


def from_file(path, mode="r"):
    with open(path, mode) as f:
        return f.read()


def save_json(path: str, obj: dict):
    to_file(json.dumps(obj), path)


def load_json(path: str):
    return json.loads(from_file(path))



from __future__ import annotations

from abc import ABCMeta

from .mixin import ToFileMixin, MetadataMixin, Dataset, Model, ObjectMixin, DatasetTransformation


class ObjectManager(ToFileMixin, MetadataMixin, ObjectMixin, metaclass=ABCMeta):
    object_manager_key = "_object_manager"

    def __init__(self, file_name=None, path=None):
        self.full_path = path, file_name
        self.metadata.update({self.object_manager_key: self.__class__.__name__})

    @ToFileMixin.file_name.setter
    def file_name(self, val) -> str:
        ToFileMixin.file_name.fset(self, val)
        self.metadata.update({"file_name": self.file_name})
        return self.file_name

    def save_file(self, path=None) -> (str, str):
        self.save_metadata(self.full_path)
        return super(ObjectManager, self).save_file(path=path)

    def load_file(self):
        super(ObjectManager, self).load_file()
        self.load_metadata(self.full_path)
        print(self.metadata)


class DatasetManager(ObjectManager, Dataset, metaclass=ABCMeta):
    pass


class ModelManager(ObjectManager, Model, metaclass=ABCMeta):
    pass


class DatasetTransformationManager(ObjectManager, DatasetTransformation, metaclass=ABCMeta):
    pass



class ToVariableWriter:
    def __init__(self):
        self._data_bytes = []

    @property
    def data_bytes(self):
        return b"".join(self._data_bytes)

    @property
    def data_str(self):
        return self.data_bytes.decode("utf-8")

    def write(self, d: bytes):
        self._data_bytes.append(d)



import os
import subprocess  # nosec
from abc import abstractmethod
from pathlib import Path
from sys import platform

from .exceptions import DecompressionError
from ..object_management.utils import ensure_path_exists


# TODO: Safety review of using subprocess for compression/decompression


class CompressionException(Exception):
    pass


base_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
pigz_binary_unix = os.path.join(base_dir, "clib", "compiled", "pigz", "pigz")
pigz_binary_win32 = os.path.join(base_dir, "clib", "pigz.exe")


class Compressor:
    @abstractmethod
    def compress(self, path, name, out_dir: str = None) -> (str, str):
        """
        e.g.
        with a sample directory structure:
        - project
            - test_data
                - datasets
                    - flowers
                        - img1.jpg
                        ...
                        - dset.csv

        compress (test_data/datasets, flowers)
        generates file:
        test_data/dataset/flowers.tar.gz
        returns (test_data/datasets, flowers.gz)
        @param path: working dir
        @param name: file or dir to compress
        @param out_dir: directory to save file to (defaults to path if None)
        @return:
        """
        raise NotImplementedError

    @abstractmethod
    def decompress(self, path, name) -> (str, str):
        raise NotImplementedError


def get_compressor() -> Compressor:
    compressors = {
        "linux": UnixCompressor,
        "linux2": UnixCompressor,
        "darwin": MacOSCompressor,
        "win32": Win32Compressor,
    }
    return compressors[platform]()


class UnixCompressor(Compressor):
    _ext = ".tar.gz"

    def compress(self, path, name, out_dir: str = None) -> (str, str):
        if not os.path.exists(os.path.join(path, name)):
            raise FileExistsError(os.path.join(path, name))
        write_dir = out_dir if out_dir is not None else path
        ensure_path_exists(write_dir)
        new_name = f"{name}{self._ext}"
        curr_dir = os.getcwd()
        command = f'cd "{path}" && tar -cf - "{name}" | {pigz_binary_unix} > "{os.path.join(write_dir, new_name)}" && cd {curr_dir}'

        with subprocess.Popen(  # nosec
            command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT  # nosec
        ) as proc:  # nosec
            cmd_out = []
            while proc.poll() is None:
                cmd_out.append(
                    proc.stdout.readline().decode()
                )  # give output from your execution/your own message
            proc.stdout.close()
            exit_code = proc.wait()
        if exit_code != 0:
            cmd_out = "\n".join(cmd_out)
            raise CompressionException(
                f"Compression error code:{exit_code}, Error Log: {cmd_out} - Please ensure pigz is installed via brew or apt"
            )  # catch return code
        return write_dir, new_name

    def decompress(self, path: str, name) -> (str, str):
        path_in = os.path.join(path, name)
        if not os.path.exists(path_in):
            raise FileExistsError(path_in)
        if not name.endswith(self._ext):
            raise DecompressionError(
                f"file name: {name} must end with {self._ext} otherwise we cannot guarantee safe decompression"
            )
        new_name = name[: -len(self._ext)]
        if not os.path.exists(path):
            Path(path).mkdir(parents=True, exist_ok=True)
        os.system(  # nosec
            f'tar --use-compress-program={pigz_binary_unix} -xf "{path_in}" -C "{path}"'  # nosec
        )  # nosec
        return path, new_name


class Win32Compressor(Compressor):
    _ext = ".tar.gz"

    def compress(self, path, name, out_dir: str = None) -> (str, str):
        if not os.path.exists(os.path.join(path, name)):
            raise FileExistsError(os.path.join(path, name))
        write_dir = out_dir if out_dir is not None else path
        ensure_path_exists(write_dir)
        new_name = f"{name}{self._ext}"
        curr_dir = os.getcwd()
        command = f'chdir "{path}" && tar -cf - "{name}" | {pigz_binary_win32} > "{os.path.join(write_dir, new_name)}" && chdir {curr_dir}'
        with subprocess.Popen(  # nosec
            command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT  # nosec
        ) as proc:  # nosec
            cmd_out = []
            while proc.poll() is None:
                cmd_out.append(
                    proc.stdout.readline().decode()
                )  # give output from your execution/your own message
            proc.stdout.close()
            exit_code = proc.wait()

        if exit_code != 0:
            cmd_out = "\n".join(cmd_out)
            raise CompressionException(
                f"Compression error code:{exit_code}, Error Log: {cmd_out} - Please ensure pigz is installed via brew or apt"
            )  # catch return code
        return write_dir, new_name

    def decompress(self, path: str, name) -> (str, str):
        path_in = os.path.join(path, name)
        if not os.path.exists(path_in):
            raise FileExistsError(path_in)
        if not name.endswith(self._ext):
            raise DecompressionError(
                f"file name: {name} must end with {self._ext} otherwise we cannot guarantee safe decompression"
            )
        new_name = name[: -len(self._ext)]
        if not os.path.exists(path):
            Path(path).mkdir(parents=True, exist_ok=True)
        os.system(  # nosec
            f'tar --use-compress-program={pigz_binary_win32} -xf "{path_in}" -C "{path}"'  # nosec
        )  # nosec
        return path, new_name


class MacOSCompressor(Compressor):
    _ext = ".tar.gz"

    def compress(self, path, name, out_dir: str = None) -> (str, str):
        if not os.path.exists(os.path.join(path, name)):
            raise FileExistsError(os.path.join(path, name))
        write_dir = out_dir if out_dir is not None else path
        ensure_path_exists(write_dir)
        new_name = f"{name}{self._ext}"
        curr_dir = os.getcwd()
        command = f'cd "{path}" && tar -cf - "{name}" | {pigz_binary_unix} > "{os.path.join(write_dir, new_name)}" && cd {curr_dir}'

        with subprocess.Popen(  # nosec
            command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT  # nosec
        ) as proc:  # nosec
            cmd_out = []
            while proc.poll() is None:
                cmd_out.append(
                    proc.stdout.readline().decode()
                )  # give output from your execution/your own message
            proc.stdout.close()
            exit_code = proc.wait()
        if exit_code != 0:
            cmd_out = "\n".join(cmd_out)
            raise CompressionException(
                f"Compression error code:{exit_code}, Error Log: {cmd_out} - Please ensure pigz is installed via brew or apt"
            )  # catch return code
        return write_dir, new_name

    def decompress(self, path: str, name) -> (str, str):
        path_in = os.path.join(path, name)
        if not os.path.exists(path_in):
            raise FileExistsError(path_in)
        if not name.endswith(self._ext):
            raise DecompressionError(
                f"file name: {name} must end with {self._ext} otherwise we cannot guarantee safe decompression"
            )
        new_name = name[: -len(self._ext)]
        if not os.path.exists(path):
            Path(path).mkdir(parents=True, exist_ok=True)
        os.system(f'tar -xf "{path_in}" -C "{path}"')  # nosec
        return path, new_name



import os
from distutils.dir_util import copy_tree

base_dir = os.path.dirname(os.path.abspath(__file__))
clib_path = os.path.join(base_dir, "clib")
compiled_path = os.path.join(clib_path, "compiled")


def make(lib_path, package_name):
    cdir = os.getcwd()
    unbuilt_path = os.path.join(lib_path, package_name)
    built_path = os.path.join(compiled_path, package_name)
    if not os.path.exists(built_path):
        copy_tree(unbuilt_path, built_path)
        os.chdir(built_path)
        os.system("make")  # nosec
        os.chdir(cdir)


make(clib_path, "pigz")



import pandas as pd
from pandas import DataFrame


def column_hash(dataset: DataFrame) -> int:
    """Invariant to ordering of columns/rows"""
    total = 0
    for col in dataset.columns:
        total += int(pd.util.hash_pandas_object(dataset[col]).sum())
    return total


def dataset_hash(dataset, project_id: str) -> str:
    return str(hash(column_hash(dataset) + hash(project_id)))


def get_dataset_project_hash(dataset, project) -> str:
    return str(hash(bytes(dataset)) + hash(project))


def get_dataset_hash(dataset) -> str:
    return str(hash(bytes(dataset)))


def get_project_hash(project) -> str:
    return str(hash(project))



from __future__ import annotations

import abc
import os
import uuid
from abc import abstractmethod
from typing import Any
from typing import List
from typing import Tuple

from .utils import save_json, load_json, ensure_path_exists
from ..core.compression import get_compressor


class SerializerMixin(metaclass=abc.ABCMeta):
    @staticmethod
    def get_serialized(obj, meta_list: List[SerializerMixin.__class__]) -> dict:
        ser_list = [meta.serialize(obj) for meta in meta_list]
        result = dict()
        for s in ser_list:
            result.update(s)
        return result

    def deserialize(self, obj: dict) -> Any:
        for key, val in obj.items():
            setattr(self, key, val)
        return self

    @abstractmethod
    def serialize(self):
        raise NotImplementedError


class DescriptionMixin(SerializerMixin):
    _description: str = None

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, val):
        self._description = val

    def serialize(self):
        return {"description": self.description}


class AttrDatasetMixin(SerializerMixin):
    _dataset: str = None

    @property
    def dataset(self):
        return self._dataset

    @dataset.setter
    def dataset(self, val):
        self._dataset = val

    def serialize(self):
        return {"dataset": self.dataset}


class AttrProjectMixin(SerializerMixin):
    _project: str = None

    @property
    def project(self):
        return self._project

    @project.setter
    def project(self, val):
        self._project = val

    def serialize(self):
        return {"project": self.project}


class AttrFrameworkMixin(SerializerMixin):
    _framework: str = None

    @property
    def framework(self):
        return self._framework

    @framework.setter
    def framework(self, val):
        self._framework = val

    def serialize(self):
        return {"framework": self.framework}


class AttrMetadataMixin(SerializerMixin):
    _metadata: dict = None

    @property
    def metadata(self) -> dict:
        """
        Object metadata which may not be saved by framework used must be of type Dict
        Empty dict if none specified.
        @return: dict
        """
        if self._metadata is None:
            self.metadata = dict()
        return self._metadata

    @metadata.setter
    def metadata(self, metadata: dict):
        """
        Metadata to be saved and loaded with the file
        @param metadata:
        @return:
        """
        if not isinstance(metadata, dict):
            raise TypeError(f"metadata of type: {type(metadata)} must be a dict")
        setattr(self, "_metadata", metadata)

    def serialize(self):
        return {"metadata": self.metadata}


class NameMixin(SerializerMixin):
    _name: str = None

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, val):
        self._name = val

    def serialize(self):
        return {"name": self.name}


# TODO: for now this is assumed that each model has a uuid, this needs to be abstracted to pk and inherit the primary key of each model.
class IDMixin(SerializerMixin):
    _uuid: str = None

    @property
    def uuid(self):
        return self._uuid

    @uuid.setter
    def uuid(self, val):
        self._uuid = val

    def serialize(self):
        return {"uuid": self.uuid}


class HashMixin(SerializerMixin):
    _hash: str = None

    @property
    def hash(self):
        return self._hash

    @hash.setter
    def hash(self, val):
        self._hash = val

    def serialize(self):
        return {"hash": self.hash}


class UsernameMixin(SerializerMixin):
    _username: str = None

    @property
    def username(self):
        return self._username

    @username.setter
    def username(self, val):
        self._username = val

    def serialize(self):
        return {"username": self.username}


class EmailMixin(SerializerMixin):
    _email: str = None

    @property
    def email(self):
        return self._email

    @email.setter
    def email(self, val):
        self._email = val

    def serialize(self):
        return {"email": self.email}


class PasswordMixin(SerializerMixin):
    _password: str = None

    @property
    def password(self):
        return self._password

    @password.setter
    def password(self, val):
        self._password = val

    def serialize(self):
        return {"password": self.password}


class ToFileMixin:
    # _controller: SecleaAI
    _file_name: str = None
    _path: str = None

    @property
    def path(self):
        if self._path is None:
            self.path = None  # let setter assign default path
        return self._path

    @path.setter
    def path(self, val: str):
        val = val if val is not None else os.getcwd()
        ensure_path_exists(val)
        self._path = val

    @property
    def full_path(self):
        return os.path.join(self.path, self.file_name)

    @full_path.setter
    def full_path(self, val: Tuple[str, str]):
        self.path, self.file_name = val

    @property
    def file_name(self) -> str:
        """
        File name under which all object data is saved
        @return: file_name in metadata, generates uuid4 if none specified
        """
        if self._file_name is None:
            self.file_name = None  # let setter generate new name.
        return self._file_name

    @file_name.setter
    def file_name(self, val: str):
        """
        File name setter, if val is None uuid4.hex is set
        @param val:
        @return:
        """
        val = val if val is not None else uuid.uuid4().__str__()
        self._file_name = val

    def save_file(self, path=None) -> (str, str):
        """
        Save tracked object to a file,
        @return: (path, file_name)
        """
        path_file_name = get_compressor().compress(self.path, self.file_name, out_dir=path)
        return path_file_name

    def load_file(self):
        """
        @return:
        """
        self.path, self.file_name = get_compressor().decompress(self.path, self.file_name)


class MetadataMixin(AttrMetadataMixin):
    _metadata_file_name = ".metadata.json"

    def save_metadata(self, path):
        ensure_path_exists(path)
        save_json(os.path.join(path, self._metadata_file_name), self.metadata)

    def load_metadata(self, path):
        self.metadata = load_json(os.path.join(path, self._metadata_file_name))


class ObjectMixin:
    _object_file_name = "object"

    @property
    @abstractmethod
    def full_path(self):
        raise NotImplementedError

    @property
    def object_file_path(self):
        return os.path.join(self.full_path, self._object_file_name)

    @abstractmethod
    def save_object(self, obj: Any):
        raise NotImplementedError

    @abstractmethod
    def load_object(self) -> Any:
        raise NotImplementedError

    @abstractmethod
    def hash_object(self, obj) -> int:
        raise NotImplementedError

    def hash_object_with_project(self, obj, project) -> int:
        return hash((self.hash_object(obj), project))


class BaseModel(IDMixin, SerializerMixin):
    def __init__(self, **kwargs):
        print(f"Object initializing {self.__class__.__name__} : {kwargs.keys()}")
        self.deserialize(kwargs)

    @abstractmethod
    def serialize(self):
        pass


class Dataset(
    BaseModel,
    NameMixin,
    DescriptionMixin,
    AttrMetadataMixin,
    AttrDatasetMixin,
    HashMixin,
    AttrProjectMixin,
):
    def serialize(self):
        return self.get_serialized(
            self,
            [NameMixin, DescriptionMixin, IDMixin, AttrMetadataMixin, HashMixin, AttrProjectMixin],
        )

    def deserialize(self, obj: dict) -> Any:
        print("attempting deserialization: ")
        r = SerializerMixin.deserialize(self, obj)
        print("deserialization successful")
        return r


class Organization(BaseModel, NameMixin):
    def serialize(self):
        return {**self.get_serialized(self, [IDMixin, NameMixin])}


class Project(BaseModel, NameMixin, DescriptionMixin):
    organization: Organization.uuid

    def serialize(self):
        return {
            **self.get_serialized(self, [IDMixin, NameMixin]),
            "organization": self.organization,
        }


class User(BaseModel, UsernameMixin, PasswordMixin, EmailMixin):
    def serialize(self):
        return self.get_serialized(self, [IDMixin, UsernameMixin, PasswordMixin, EmailMixin])


class DatasetTransformation(BaseModel, NameMixin, DescriptionMixin):
    def serialize(self):
        return self.get_serialized(self, [NameMixin, DescriptionMixin, IDMixin])


class Model(BaseModel, NameMixin, DescriptionMixin, AttrFrameworkMixin):
    _model_name_key = "_model_name"

    def serialize(self):
        return self.get_serialized(self, [NameMixin, DescriptionMixin])

    @staticmethod
    @abstractmethod
    def get_params(model) -> dict:
        """
        Extract parameters of model
        @param model:
        @return:
        """
        pass


class ModelState(BaseModel, NameMixin, DescriptionMixin):
    def serialize(self):
        return self.get_serialized(self, [NameMixin, DescriptionMixin, IDMixin])


class TrainingRun(BaseModel, NameMixin, DescriptionMixin):
    def serialize(self):
        return self.get_serialized(self, [NameMixin, DescriptionMixin, IDMixin])



from __future__ import annotations

import inspect
import os
import re
from shutil import rmtree
from typing import TypeVar

from decorator import decorate
from wrapt.wrappers import ObjectProxy

from .factory import load_object_manager_from_class_name
from .factory import load_object_manager_from_obj
from .object_manager import ObjectManager
from .utils import ensure_path_exists

R = TypeVar("R")  # the variable return type

DEF = re.compile(r"\s*def\s*([_\w][_\w\d]*)\s*\(")
POS = inspect.Parameter.POSITIONAL_OR_KEYWORD
EMPTY = inspect.Parameter.empty


class Tracked(ObjectProxy):
    _object_manager: ObjectManager = None

    def __init__(self, wrapped, type_hint=None, cleanup: bool = True):
        # Initialise the object proxy and assign variables for object.
        super(Tracked, self).__init__(wrapped)
        self._cleanup = cleanup

    @property
    def object_manager(self) -> ObjectManager:
        if self._object_manager is None:
            try:
                om = load_object_manager_from_obj(self)
            except Exception as e:
                print(f"failed to load proxy: {e}")
                raise
            self._object_manager = om
        return self._object_manager

    @staticmethod
    def load_tracked(path, file_name, cleanup: bool = True) -> Tracked:
        """
        Load routine for tracked object
        @param path:
        @param file_name:
        @return:
        """
        # use generic object manager to load the file (fil compressed etc.
        generic_object_manager = ObjectManager(file_name=file_name, path=path)
        generic_object_manager.load_file()
        #  get the specific object manager for the saved object
        object_manager = load_object_manager_from_class_name(
            generic_object_manager.metadata.get(generic_object_manager.object_manager_key)
        )
        object_manager.metadata.update(generic_object_manager.metadata)
        object_manager.full_path = generic_object_manager.path, generic_object_manager.file_name
        # load the tracked object
        obj = object_manager.load_object()
        # wrap object, and assign previously loaded object_manager.
        tracked_obj = Tracked(obj, cleanup=cleanup)
        tracked_obj._object_manager = object_manager
        return tracked_obj

    def save_tracked(self, path=None) -> (str, str):
        """
        Save routine for tracked object, path: directory to save object to
        @return: save_path, file_name
        """
        ensure_path_exists(os.path.join(self.object_manager.full_path))
        self.object_manager.save_object(self)
        args = self.object_manager.save_file(path=path)
        # For now as the object is now saved to a zip, we remove the file created by the tracked object.
        return args

    def __del__(self):
        # for now clean up as file management is not consistent in use.
        if not self._cleanup:
            return
        try:
            rmtree(self.object_manager.full_path)
        except FileNotFoundError:
            pass

    def __call__(self, *args, **kwargs):
        # entry
        try:
            return self.__wrapped__(*args, **kwargs)
        finally:
            # exit
            pass

    def __reduce_ex__(self, protocol):
        return self.__wrapped__.__reduce_ex__(protocol)

    def __copy__(self):
        return self.__wrapped__.__copy__()

    def __deepcopy__(self, memo):
        return self.__wrapped__.__deepcopy__()

    def __reduce__(self):
        return self.__wrapped__.__reduce__()

    def __hash__(self):
        return self.object_manager.hash_object(self.__wrapped__)


def tracked_decorator(caller, _func=None, kwsyntax=False):
    """
    decorator(caller) converts a caller function into a decorator
    """
    if _func is not None:  # return a decorated function
        # this is obsolete behavior; you should use decorate instead
        return decorate(_func, caller, (), kwsyntax)
    # else return a decorator function
    sig = inspect.signature(caller)
    dec_params = [p for p in sig.parameters.values() if p.kind is POS]

    def dec(func=None, *args, **kw):
        na = len(args) + 1
        extras = args + tuple(
            kw.get(p.name, p.default) for p in dec_params[na:] if p.default is not EMPTY
        )
        if func is None:
            return lambda func: decorate(func, caller, extras, kwsyntax)
        else:
            return decorate(func, caller, extras, kwsyntax)

    dec.__signature__ = sig.replace(parameters=dec_params)
    dec.__name__ = caller.__name__
    dec.__doc__ = caller.__doc__
    dec.__wrapped__ = caller
    dec.__qualname__ = caller.__qualname__
    dec.__kwdefaults__ = getattr(caller, "__kwdefaults__", None)
    dec.__dict__.update(caller.__dict__)

    # TODO make this generic for multiple types of functions this is a hacky workarount.
    #  Assume all functions using this are generic dataset transformation functions as soon as we need specific types
    #  this needs taking out and refactoring
    @dec
    def ret(f, *args, **kwargs):
        # here we can upload dataset transformation
        return f(args, kwargs)

    return Tracked(ret)



from types import FunctionType
from typing import Any
from typing import List, Tuple

from .dataset_transformation.dataset_transformation import GenericDatasetTransformationManager
from .object_manager import ObjectManager


# TODO: import model managers and types directly from their respective modules?
# i.e. find a way not to list everything
def _load_tf_managers():
    try:
        import tensorflow as tf
        from .model.tensorflow_model import TensorflowModelManager
        from .dataset.tensorflow_dataset import TensorflowDatasetManager
    except ImportError:
        print("tensorflow import not found, tensorflow managers will not be loaded")
        return []
    return [
        (tf.data.Dataset, TensorflowDatasetManager),
        (tf.keras.Sequential, TensorflowModelManager),
    ]


def _load_lgbm_managers():
    try:
        from .model.lightgbm_model import LightGBMModelManager
        import lightgbm as lgb
    except ImportError:
        print("lightgbm import not found, lightgbm managers will not be loaded")
        return []
    return [
        (lgb.Booster, LightGBMModelManager),
        (lgb.LGBMModel, LightGBMModelManager),
        (lgb.LGBMRanker, LightGBMModelManager),
        (lgb.LGBMRegressor, LightGBMModelManager),
        (lgb.LGBMClassifier, LightGBMModelManager),
        (lgb.DaskLGBMRanker, LightGBMModelManager),
        (lgb.DaskLGBMRegressor, LightGBMModelManager),
        (lgb.DaskLGBMClassifier, LightGBMModelManager),
    ]


def _load_sklearn_managers():
    try:
        from sklearn.base import BaseEstimator
        from .model.sklearn_model import SKLearnModelManager
    except ImportError:
        print("sklearn import not found, sklearn managers will not be loaded")
        return []
    return [
        (BaseEstimator, SKLearnModelManager),
    ]


def _load_xgb_managers():
    try:
        import xgboost as xgb
        from .model.xgb_model import XGBoostModelManager
    except ImportError:
        print("xgboost import not found, xgboost managers will not be loaded")
        return []
    return [
        (xgb.XGBClassifier, XGBoostModelManager),
        (xgb.Booster, XGBoostModelManager),
        (xgb.XGBModel, XGBoostModelManager),
        (xgb.XGBRanker, XGBoostModelManager),
        (xgb.XGBRegressor, XGBoostModelManager),
        (xgb.XGBRFRegressor, XGBoostModelManager),
        (xgb.XGBRFClassifier, XGBoostModelManager),
    ]


def _load_pandas_managers():
    from .dataset.pandas_dataset import PandasDatasetManager
    import pandas as pd

    return [
        (pd.DataFrame, PandasDatasetManager),
    ]


def _get_object_manager_class_types():
    # Needs to be imported lazilly as it returns a tracked object type
    from .proxy import tracked_decorator

    _object_manager_class_types: List[Tuple[type, ObjectManager.__class__]] = [
        (tracked_decorator, GenericDatasetTransformationManager),
        *_load_tf_managers(),
        *_load_lgbm_managers(),
        *_load_xgb_managers(),
        *_load_pandas_managers(),
        *_load_sklearn_managers(),
    ]

    _combined_object_managers_mappings = {
        manager.__name__: manager for t, manager in _object_manager_class_types
    }

    return _object_manager_class_types, _combined_object_managers_mappings


def load_object_manager_from_class_name(class_name: str) -> ObjectManager:
    _, mapper = _get_object_manager_class_types()
    if class_name not in mapper:
        raise NotImplementedError(f"No object manager found for class: {class_name}")
    return mapper.get(class_name)()


def load_object_manager_from_obj(obj: Any) -> ObjectManager:
    type_manager_list, _ = _get_object_manager_class_types()
    for t, manager in type_manager_list:
        try:
            if isinstance(obj, FunctionType) and isinstance(t, FunctionType):
                if obj.__name__ == t.__name__:
                    return manager()
            elif not isinstance(obj, FunctionType) and not isinstance(t, FunctionType):
                if isinstance(obj, t):
                    return manager()
        except Exception as e:
            print(
                f"Exception checking type: {t} with manager: {manager} during load proxy object manager: {e}"
            )
    raise NotImplementedError(f"Not object manager for object: {obj.__class__}")



import os.path
import unittest

from ...core.compression import get_compressor

base_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


class TestOsCompression(unittest.TestCase):
    def setUp(self) -> None:
        self.comp = get_compressor()

    def test_compress_decompress_file(self):
        d_path, d_name = f"{base_dir}/example_files", "bee_movie.txt"
        o_path = os.path.join(d_path, d_name)
        orig_size = os.path.getsize(o_path)
        c_path, c_name = self.comp.compress(d_path, d_name)
        self.assertTrue(os.path.exists(os.path.join(c_path, c_name)))
        self.assertLess(
            os.path.getsize(os.path.join(c_path, c_name)),
            orig_size,
            "File post compression not smaller",
        )
        d_p, d_n = self.comp.decompress(c_path, c_name)
        self.assertEqual(
            os.path.getsize(os.path.join(d_p, d_n)),
            orig_size,
            "File post decompression not original size",
        )
        os.remove(os.path.join(c_path, c_name))

    def test_compress_decompress_dir(self):
        d_path, d_name = f"{base_dir}/data", "compress_example_dir"
        o_path = os.path.join(d_path, d_name)
        orig_size = os.path.getsize(os.path.join(o_path, "data")) + os.path.getsize(
            os.path.join(o_path, "object")
        )
        c_path, c_name = self.comp.compress(d_path, d_name)
        self.assertTrue(os.path.exists(os.path.join(c_path, c_name)))
        self.assertLess(
            os.path.getsize(os.path.join(c_path, c_name)),
            orig_size,
            "File post compression not smaller",
        )
        d_p, d_n = self.comp.decompress(c_path, c_name)
        d_path = os.path.join(d_p, d_n)
        d_size = os.path.getsize(os.path.join(d_path, "data")) + os.path.getsize(
            os.path.join(d_path, "object")
        )
        self.assertEqual(
            d_size,
            orig_size,
            "File post decompression not original size",
        )
        # os.remove(os.path.join(c_path, c_name))


if __name__ == "__main__":
    unittest.main()
